require 'test/lint'
require 'formality'

::Question = Struct.new(:id, :text, :answers) do
  def attributes; Hash[members.zip(values)] end
end

::Answer = Struct.new(:id, :text) do
  def attributes; Hash[members.zip(values)] end
end

class ::QuestionForm
  include Formality
  attribute :text
  nest_many :answer_forms, :from_model_attribute => :answers
  validates_presence_of :text, :message => "can't be empty"
end

class ::AnswerForm
  include Formality
  attribute :text
  validates_presence_of :text, :message => "can't be empty"
end

lint(AnswerForm.new)
lint(QuestionForm.new)

tst ".nest_many: defines an '<assoc>' reader" do
  form = QuestionForm.new
  assert form.respond_to?(:answer_forms)
end

tst ".nest_many: defines an '<assoc>_attributes' reader" do
  form = QuestionForm.new
  assert form.respond_to?(:answer_forms_attributes)
end

tst ".nest_many: reader defaults to an empty array" do
  form = QuestionForm.new
  assert_equal [], form.answer_forms
end

tst ".nest_many: defines an '<assoc>_attributes=' writer" do
  form = QuestionForm.new
  assert form.respond_to?(:answer_forms_attributes=)
end

tst ".nest_many: reader returns an Array of child form objects" do
  form = QuestionForm.new
  form.answer_forms_attributes = [{:text => "first"}]
  answer_form = form.answer_forms.first

  assert answer_form.kind_of?(AnswerForm)
  assert_equal "first", answer_form.text
end

tst "#attributes: includes the nested form's attributes" do
  form = QuestionForm.new
  form.assign :answer_forms_attributes => [{:text => "first"}]


  expected = {"text" => nil,
              "answer_forms_attributes" => [{"text" => "first"}]}

  assert_equal expected, form.attributes
end

tst "#valid?: returns false if any nested forms are invalid" do
  form = QuestionForm.new.assign :text => "question text"
  form.answer_forms_attributes = [{:text => ""}]

  assert_equal false, form.valid?
end

tst "valid?: sets errors on the nested forms" do
  form = QuestionForm.new.assign :text => "question text"
  form.answer_forms_attributes = [{:text => ""}]
  form.valid?
  answer_form = form.answer_forms.first

  assert_equal ["can't be empty"], answer_form.errors[:text]
end

tst "#invalid?: returns true if any nested forms are invalid" do
  form = QuestionForm.new.assign :text => "question text"
  form.answer_forms_attributes = [{:text => ""}]

  assert_equal true, form.invalid?
end

tst "#invalid?: sets errors on the nested forms" do
  form = QuestionForm.new.assign :text => "question text"
  form.answer_forms_attributes = [{:text => ""}]
  form.invalid?
  answer_form = form.answer_forms.first

  assert_equal ["can't be empty"], answer_form.errors[:text]
end

tst "#from_model: assigns to nested form from nested models" do
  question = Question.new(1, "Question 1")
  question.answers = [Answer.new(1, "Answer 1"), Answer.new(2, "Answer 2")]
  form = QuestionForm.from_model(question)

  answer_form_1, answer_form_2 = form.answer_forms

  assert_equal 1, answer_form_1.id
  assert_equal "Answer 1", answer_form_1.text
  assert_equal 2, answer_form_2.id
  assert_equal "Answer 2", answer_form_2.text
end

Object.send(:remove_const, :QuestionForm)
Object.send(:remove_const, :AnswerForm)
Object.send(:remove_const, :Question)
Object.send(:remove_const, :Answer)
