require 'test/lint'
require 'formality'

::Question = Struct.new(:id, :text, :answer) do
  def attributes; Hash[members.zip(values)] end
end

::Answer = Struct.new(:id, :text) do
  def attributes; Hash[members.zip(values)] end
end


class ::QuestionForm
  include Formality
  attribute :text
  nest_one :answer_form, :from_model_attribute => :answer
  validates_presence_of :text, :message => "can't be empty"
end

class ::AnswerForm
  include Formality
  attribute :text
  validates_presence_of :text, :message => "can't be empty"
end

tst ".nest_one: defines an '<assoc>' reader" do
  form = QuestionForm.new
  assert form.respond_to?(:answer_form)
end

tst ".nest_one: defines an '<assoc>_attributes' reader" do
  form = QuestionForm.new
  assert form.respond_to?(:answer_form_attributes)
end

tst ".nest_one: reader defaults to nil" do
  form = QuestionForm.new
  assert_equal nil, form.answer_form
end

tst ".nest_one: defines an '<assoc>_attributes=' writer" do
  form = QuestionForm.new
  assert form.respond_to?(:answer_form_attributes=)
end

tst ".nest_many: reader returns a child form object" do
  form = QuestionForm.new
  form.answer_form_attributes = {:text => "first"}

  assert form.answer_form.kind_of?(AnswerForm)
  assert_equal "first", form.answer_form.text
end

tst "#attributes: includes the nested form's attributes" do
  form = QuestionForm.new
  form.answer_form_attributes = {:text => "first"}

  expected = {"text" => nil,
              "answer_form_attributes" => {"text" => "first"}}

  assert_equal expected, form.attributes
end

tst "#valid?: returns false if the nested form is invalid" do
  form = QuestionForm.new.assign :text => "question text"
  form.answer_form_attributes = {:text => ""}

  assert_equal false, form.valid?
end

tst "valid?: sets errors on the nested form" do
  form = QuestionForm.new.assign :text => "question text"
  form.answer_form_attributes = {:text => ""}
  form.valid?

  assert_equal ["can't be empty"], form.answer_form.errors[:text]
end

tst "#invalid?: returns true if the nested form is invalid" do
  form = QuestionForm.new.assign :text => "question text"
  form.answer_form_attributes = {:text => ""}

  assert_equal true, form.invalid?
end

tst "invalid?: sets errors on the nested form" do
  form = QuestionForm.new.assign :text => "question text"
  form.answer_form_attributes = {:text => ""}
  form.invalid?

  assert_equal ["can't be empty"], form.answer_form.errors[:text]
end

tst "#from_model: assigns to nested form from nested model" do
  question = Question.new(1, "Question 1")
  question.answer = Answer.new(1, "Answer 1")
  form = QuestionForm.from_model(question)

  assert_equal 1, form.answer_form.id
  assert_equal "Answer 1", form.answer_form.text
end

Object.send(:remove_const, :QuestionForm)
Object.send(:remove_const, :AnswerForm)
Object.send(:remove_const, :Question)
Object.send(:remove_const, :Answer)
