require 'test/lint'
require 'formality'

# A form class with validations
class ::LoginForm
  include Formality

  attribute :email
  attribute :password
  attribute :password_confirmation

  validates_presence_of :email, :password, :password_confirmation,
    :message => "can't be empty"

  validates_confirmation_of :password,
    :message => "don't match"
end

# Check that we still pass ActiveModel::Lint
lint(LoginForm.new)

# A little helper to generate valid attributes
def valid_attributes
  { :email => 'test@example.com',
    :password => '123',
    :password_confirmation => '123' }
end

tst "the builtin ActiveModel validations exist" do
  [ :validates, :validates!, :validate, :validates_each,
    :validates_confirmation_of, :validates_exclusion_of,
    :validates_inclusion_of, :validates_format_of, :validates_length_of,
    :validates_numericality_of, :validates_presence_of, :validates_with
  ].each do |validator|
    assert LoginForm.respond_to?(validator)
  end
end

tst "#valid? and #invalid? do the right thing when conditions ARE NOT met" do
  form = LoginForm.new
  assert_equal false, form.valid?
  assert_equal true, form.invalid?
end

tst "#valid? and #invalid? do the right thing when conditions ARE met" do
  form = LoginForm.new.assign(valid_attributes)
  assert_equal true, form.valid?
  assert_equal false, form.invalid?
end

tst "#errors contains the error messages" do
  form = LoginForm.new

  assert_equal false, form.valid?
  assert_equal ["can't be empty"], form.errors[:email]
  assert_equal ["can't be empty"], form.errors[:password]
  assert_equal ["can't be empty"], form.errors[:password_confirmation]

  form.email = "test@example.com"
  form.password = "123"
  form.password_confirmation = "1234"

  assert_equal false, form.valid?
  assert_equal ["don't match"], form.errors[:password]
end

tst "#valid runs its block if the form is valid" do
  form = LoginForm.new.assign(valid_attributes)
  called = 0
  form.valid { called += 1 }
  assert_equal 1, called
end

tst "#valid does NOT run its block if the form is invalid" do
  form = LoginForm.new
  form.valid { raise("it's an error if you get here") }
end

tst "#invalid runs its block if the form is invalid" do
  form = LoginForm.new
  called = 0
  form.invalid { called += 1 }
  assert_equal 1, called
end

tst "#invalid does NOT run its block if the form is valid" do
  form = LoginForm.new.assign(valid_attributes)
  form.invalid { raise("it's an error if you get here") }
end

Object.send(:remove_const, :LoginForm)
